/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.awt.Dimension;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import javax.swing.*;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.model.FieldPlan;
import nl.cloudfarming.client.model.PartField;
import nl.cloudfarming.client.model.crop.Crop;
import nl.cloudfarming.client.model.crop.Race;
import org.jdesktop.swingx.JXDatePicker;
import org.jdesktop.swingx.JXPanel;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;

/**
 * Panel to plan a new ProductionUnit.
 * @author Gerben Feenstra
 */
public class PlanProductionUnitPanel extends JXPanel {

    // Static Variables
    private static final String LABEL_NAME = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit.panel.label.name");
    private static final String LABEL_CROP = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit.panel.label.crop");
    private static final String LABEL_STARTDATE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit.panel.label.startdate");
    private static final String LABEL_ENDDATE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit.panel.label.enddate");
    private static final String LABEL_PARTFIELD_LIST = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit.panel.label.partFields");
    private static final String MESSAGE_CROP_HAS_NO_RACES = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit_action.combobox.no_races_message");
    private static final String MESSAGE_NO_CROPS_AVAILABLE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.plan_production_unit_action.combobox.no_crops_message");    
    
    // Combobox identifiers
    private static final String COMBOBOX_RACES_NAME = "racescombobox";
    private static final String COMBOBOX_CROPS_NAME = "cropscombobox";
    
    // Services
    private TacticalPlanningService tacticalPlanningService = Lookup.getDefault().lookup(TacticalPlanningService.class);
    
    // Panel components
    private final JTextField nameTextField = new JTextField();
    private final JXDatePicker startDatePicker = new JXDatePicker();
    private final JXDatePicker endDatePicker = new JXDatePicker();
    private final JComboBox cropsComboBox;
    private final JComboBox racesComboBox;
    private JList unoccupiedPartFieldsList;
    
    // Other variables
    PlanProductionUnitController controller;

    /**
     * Constructor
     * Create an instance of this action
     */
    public PlanProductionUnitPanel(PlanProductionUnitController controller, FieldPlan fieldPlan) {
        this.controller = controller;
        //
        // populate the panel with the information about what the user is about to save
        //
        this.setLayout(new MigLayout("wrap 3"));
        //
        // construct the comboboxes
        //
        cropsComboBox = new JComboBox();
        cropsComboBox.setName(COMBOBOX_CROPS_NAME);
        racesComboBox = new JComboBox();
        racesComboBox.setName(COMBOBOX_RACES_NAME);
        racesComboBox.setPreferredSize(new Dimension(150, 0));
        populateCrops(null);
        populateRaces(null);

        JScrollPane fieldScrollPane = getFieldScrollPane(fieldPlan);

        // Dirty dimension setting hack...
        nameTextField.setPreferredSize(new Dimension(fieldScrollPane.getPreferredSize().width, 0));

        // PU Name
        this.add(new JLabel(LABEL_NAME));
        this.add(nameTextField, "span");

        // Crop/race comboboxes
        this.add(new JLabel(LABEL_CROP));
        this.add(cropsComboBox);
        this.add(racesComboBox);

        // Start date
        this.add(new JLabel(LABEL_STARTDATE));
        this.add(startDatePicker, "span");

        // End date
        this.add(new JLabel(LABEL_ENDDATE));
        this.add(endDatePicker, "span");

        // Fields
        this.add(new JLabel(LABEL_PARTFIELD_LIST));
        this.add(fieldScrollPane, "span");
    }

    public final void populateRaces(List<Race> races){
        
        
        if(races == null || races.isEmpty()){
            racesComboBox.removeAllItems();
            racesComboBox.addItem(MESSAGE_CROP_HAS_NO_RACES);
            racesComboBox.setEnabled(false);
        }else{
            Collections.sort(races);    
            racesComboBox.removeAllItems();
            for(Race race:races){
                racesComboBox.addItem(race);
            }
            racesComboBox.setSelectedIndex(0);
            racesComboBox.setEnabled(true);
        }
    }
    
    public final void populateCrops(List<Crop> crops){
        if(crops == null || crops.isEmpty()){
            cropsComboBox.removeAllItems();
            cropsComboBox.addItem(MESSAGE_NO_CROPS_AVAILABLE);
            cropsComboBox.setEnabled(false);
        }else{
            Collections.sort(crops);
            cropsComboBox.removeAllItems();
            for(Crop crop:crops){
                cropsComboBox.addItem(crop);
            }
            cropsComboBox.setSelectedIndex(0);
            cropsComboBox.setEnabled(true);
        }        
    }    


    /**
     * Create scrollpane containing all Fields.
     * 
     * @return Scrollpane containing all Fields
     */
    private JScrollPane getFieldScrollPane(FieldPlan fieldPlan) {
        JScrollPane scrollPane = new JScrollPane();
        Collection<PartField> partFields = tacticalPlanningService.findUnoccupiedPartFields(fieldPlan);

        unoccupiedPartFieldsList = new JList();
        
        if(partFields != null && !partFields.isEmpty()){
            unoccupiedPartFieldsList.setListData(partFields.toArray());
        }

        scrollPane.setViewportView(unoccupiedPartFieldsList);
        return scrollPane;
    }

    public JComboBox getCropsComboBox() {
        return cropsComboBox;
    }

    public JComboBox getRacesComboBox() {
        return racesComboBox;
    }
    
    
    
    /**
     * 
     * @return Fields JList
     */
    public JList getAvailableFieldsList(){
        return unoccupiedPartFieldsList;
    }
    
    /**
     * 
     * @return The startdate picker
     */
    public JXDatePicker getStartDatePicker(){
        return this.startDatePicker;
    }
    
    /**
     * 
     * @return The enddate picker
     */
    public JXDatePicker getEndDatePicker(){
        return this.endDatePicker;
    }
    
    /**
     * 
     * @return The name textfield
     */
    public JTextField getNameTextField(){
        return nameTextField;
    }            
}