/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.awt.Color;
import java.awt.datatransfer.Transferable;
import java.beans.IntrospectionException;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.*;
import nl.cloudfarming.client.planning.tactical.wizard.PlanningWizard;
import org.apache.commons.lang.StringUtils;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.lookup.Lookups;

/**
 *
 * @author Timon Veenstra
 */
class ProductionUnitManager {

    private final ProductionUnit productionUnit;
    private final Node rootNode;
    private final List<PartField> partFields = Collections.synchronizedList(new ArrayList<PartField>());
    private final ConcurrentMap<Operation, OperationManager> operationMap = new ConcurrentHashMap<Operation, OperationManager>();
    private final ProductionUnitLayer layer;
    private final PartFieldNodeFactory partFieldNodeFactory;
    private final OperationNodeFactory operationNodeFactory;
    private final AppLogger LOG = AppLogFactory.getLogger(ProductionUnitManager.class);

    public ProductionUnitManager(ProductionUnit productionUnit) {
        assert productionUnit != null : "Production unit is required argument and cannot be null";
        this.productionUnit = productionUnit;
        layer = new ProductionUnitLayer(new Palette(Color.BLACK), productionUnit);
        partFieldNodeFactory = new PartFieldNodeFactory();
        operationNodeFactory = new OperationNodeFactory();
        
        try {
            rootNode = new ProductionUnitNode();
        } catch (IntrospectionException ex) {
            Exceptions.printStackTrace(ex);
            throw new RuntimeException(ex);
        }
        
        // Testing purposes
        Operation dummy1 = new Operation();
        OperationManager dummyManager1 = new OperationManager(dummy1);
        operationMap.put(dummy1, dummyManager1);
        
        // Testing purposes
        Operation dummy2 = new Operation();
        OperationManager dummyManager2 = new OperationManager(dummy2);
        operationMap.put(dummy2, dummyManager2);
    }

    Node getRootNode() {
        return rootNode;
    }

    boolean contains(PartField partField) {
        return getPartFields().contains(partField);
    }

    List<PartField> getPartFields() {
        return partFields;
    }

    OperationManager addOperation(Operation operation) {
        OperationManager operationManager = new OperationManager(operation);
        operationMap.put(operation, operationManager);
        operationNodeFactory.refresh();
        return operationManager;
    }

    ConcurrentMap<Operation, OperationManager> getOperations() {
        return operationMap;
    }

    void addPartField(PartField partField) {
        partFields.add(partField);
        partFieldNodeFactory.refresh();

    }

    void removePartField(PartField partField) {
        partFields.remove(partField);
        partFieldNodeFactory.refresh();
    }
   
    class ProductionUnitChildFactory extends ChildFactory<Node>{

        @Override
        protected Node createNodeForKey(Node key) {
            return key;
        }
        
        @Override
        protected boolean createKeys(List<Node> toPopulate) {
            Node partFieldsNode = new IconNode(Children.create(partFieldNodeFactory, true),"nl/cloudfarming/client/planning/tactical/map-icon.png");
            //FIXME bundle
            partFieldsNode.setDisplayName("partfields");
            toPopulate.add(partFieldsNode);
            
            //FIXME icon
            Node operationsNode = new IconNode(Children.create(operationNodeFactory, true),"");
            //FIXME bundle
            operationsNode.setDisplayName("operations");
            toPopulate.add(operationsNode);
            
            return true;
        }
        
    }

    class ProductionUnitNode extends AgroSenseBeanNode<ProductionUnit> {

        public ProductionUnitNode() throws IntrospectionException {
            super(productionUnit, Children.create(new ProductionUnitChildFactory(), true), Lookups.fixed(productionUnit, layer));

            productionUnit.addPropertyChangeListener(ProductionUnit.PROP_CROP, new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    fireDisplayNameChange(getName(), getDisplayName());
                }
            });

            productionUnit.addPropertyChangeListener(ProductionUnit.PROP_COLOR, new PropertyChangeListener() {

                @Override
                public void propertyChange(PropertyChangeEvent evt) {
                    layer.getPalette().setBaseColor((Color) evt.getNewValue());
                    fireDisplayNameChange(getDisplayName(), getHtmlDisplayName());
                }
            });

            // FIXME: PU icon.
            setIconBaseWithExtension(Layer.LAYER_NODE_ICON_BASE);
        }

        @Override
        public String getHtmlDisplayName() {
            StringBuilder sb = new StringBuilder("<font color=\"#");
            Color color = (getBean().getColor() != null) ? getBean().getColor() : Color.BLACK;
            sb.append(StringUtils.leftPad(Integer.toHexString(color.getRed()), 2, '0'));
            sb.append(StringUtils.leftPad(Integer.toHexString(color.getGreen()), 2, '0'));
            sb.append(StringUtils.leftPad(Integer.toHexString(color.getBlue()), 2, '0'));
            sb.append("\">");
            sb.append(getDisplayName());
            sb.append("</font>");
            return sb.toString();
        }

        @Override
        public String getDisplayName() {
            if (productionUnit.getRace() != null && productionUnit.getRace().getCrop() != null) {
                StringBuilder sb = new StringBuilder(productionUnit.getName());
                sb.append(" (");
                sb.append(productionUnit.getRace().getCrop());
                sb.append(")");
                return sb.toString();
            } else {
                return getName();
            }
        }

        @Override
        public Action[] getActions(boolean context) {
            if (getPartFields().size() > 0) {
                return new Action[]{new PlanningWizard()};
            }
            return null;

        }

        /**
         * Method overriden from super. When ProductionUnit is dragged the node
         * should be expanded otherwise the partFields are nog loaded and
         * therefore not shown on the map.
         *
         * @return
         * @throws IOException
         */
        @Override
        public Transferable drag() throws IOException {
            getChildren().getNodes();
            return super.drag();
        }
    }

    class OperationNodeFactory extends ChildFactory<OperationManager> {

        @Override
        protected Node createNodeForKey(OperationManager key) {
            return key.getRootNode();
        }

        
        @Override
        protected boolean createKeys(List<OperationManager> toPopulate) {
            toPopulate.addAll(getOperations().values());
            return true;
        }

        private void refresh() {
            refresh(true);
        }        
    }
        
    class PartFieldNodeFactory extends ChildFactory<PartField> {

        /**
         * Creates Node based on bean. Name is field.node.name + id
         *
         * @param key PartField which is wrapped in this node
         * @return PartFieldNode which is created for this PartField
         */
        @Override
        protected Node createNodeForKey(PartField key) {
            try {
                return new PartFieldNode(layer, key);
            } catch (IntrospectionException ex) {
                Exceptions.printStackTrace(ex);
                return null;
            }
        }

        /**
         * Get all the keys based on all the available partFields in
         * productionUnit
         *
         * @param toPopulate List of partFields to which the new partfiels are
         * added
         * @return always true
         */
        @Override
        protected boolean createKeys(List<PartField> toPopulate) {
            toPopulate.addAll(getPartFields());
            return true;
        }

        private void refresh() {
            refresh(true);
        }
    }
}
