/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.planning.tactical;

import java.beans.IntrospectionException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.logging.Logger;
import javax.swing.Action;
import nl.cloudfarming.client.area.AreaService;
import nl.cloudfarming.client.model.*;
import nl.cloudfarming.client.model.crop.Race;
import org.openide.explorer.ExplorerManager;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.lookup.ServiceProvider;

/**
 * Persistence service for Tactical planning
 *
 * @author Timon Veenstra
 */
@ServiceProvider(service = TacticalPlanningService.class)
public final class TacticalPlanningServiceImpl extends CommonModelService implements TacticalPlanningService {

    private static final Logger LOGGER = Logger.getLogger(TacticalPlanningServiceImpl.class.getName());
    // Fieldplan explorer rootnode, contains available fieldplans as children
    private final Node rootNode;
    // concurrent map with all managed fieldplans
    private final ConcurrentMap<FieldPlan, FieldPlanManager> fieldPlanMap = new ConcurrentHashMap<FieldPlan, FieldPlanManager>();
    // childfactory for the rootnode which creates the fieldplan childnodes
    private final FieldPlanNodeFactory fieldPlanNodeFactory = new FieldPlanNodeFactory();

    public TacticalPlanningServiceImpl() {
        try {
            // initialize the root node
            rootNode = new FieldPlanRootNode();
        } catch (IntrospectionException ex) {
            Exceptions.printStackTrace(ex);
            throw new RuntimeException(ex);
        }
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public ProductionUnit createProductionUnit(FieldPlan fieldPlan, Race race, Date startDate, Date endDate) {
        ProductionUnit productionUnit = new ProductionUnit.Builder().setEndDate(endDate).setFieldPlan(fieldPlan).setRace(race).setStartDate(startDate).build();

        getManagerForFieldPlan(fieldPlan).addProductionUnit(productionUnit);

        return productionUnit;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public FieldPlan createFieldPlanForYear(String name, int year) {

        FieldPlan fieldPlan = new FieldPlan();
        fieldPlan.setPlanYear(year);
        fieldPlan.setName(name);
        addFieldPlan(fieldPlan);
        FieldPlanManager fpm = getManagerForFieldPlan(fieldPlan);

        AreaService areaService = Lookup.getDefault().lookup(AreaService.class);
        Collection<Field> fields = areaService.findAllCurrentFields();

        for (Field field : fields) {
            PartField partField = new PartField();
            partField.setField(field);
            partField.setFieldPlan(fieldPlan);
            partField.setGeometry(field.getGeometry());
            fpm.getUnoccupiedFieldsManager().addPartField(partField);
        }

        return fieldPlan;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    protected ExplorerManager initExplorer() {
        ExplorerManager manager = new ExplorerManager();
        manager.setRootContext(rootNode);
        manager.getRootContext().setDisplayName(NbBundle.getMessage(this.getClass(), "planning_explorer.rootnode"));
        return manager;
    }

    /**
     * {@inheritDoc }
     */
    @Override
    public void linkPartFieldToProductionUnit(PartField partField, ProductionUnit productionUnit) {
        FieldPlanManager fieldPlanManager = fieldPlanMap.get(productionUnit.getFieldPlan());
        ProductionUnitManager unoccupiedFieldsManager = fieldPlanManager.getUnoccupiedFieldsManager();
        if (unoccupiedFieldsManager.contains(partField)) {
            unoccupiedFieldsManager.removePartField(partField);
        } else {
            throw new UnsupportedOperationException("Linking of partfields already linked to a production unit is not supported yet.");
        }
        fieldPlanManager.getManagerForProductionUnit(productionUnit).addPartField(partField);


    }

    /**
     * {@inheritDoc }
     */
    @Override
    public Operation planOperation(ProductionUnit productionUnit, Operation.Type type, Date planDate) {
        assert productionUnit != null : "production unit is a required parameted and cannot be null";
        Operation operation = new Operation();
        operation.setType(type);
        operation.setProductionUnit(productionUnit);
        operation.setPlanDate(planDate);
        return operation;
    }

    /**
     * Add a fieldplan to the managed fieldplans.
     * if the fieldplan is already managed nothing happens
     * 
     * @param fieldPlan 
     */
    private void addFieldPlan(FieldPlan fieldPlan) {
        fieldPlanMap.putIfAbsent(fieldPlan, new FieldPlanManager(fieldPlan));
        fieldPlanNodeFactory.refresh();
    }

    /**
     * Get the fieldplan manager for the specified fieldplan
     * 
     * @param fieldPlan
     * @return Fieldplan manager, null if the fieldplan is not managed
     */
    private FieldPlanManager getManagerForFieldPlan(FieldPlan fieldPlan) {
        return fieldPlanMap.get(fieldPlan);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Collection<PartField> findUnoccupiedPartFields(FieldPlan fieldPlan) {
        FieldPlanManager fpm = getManagerForFieldPlan(fieldPlan);
        return fpm.getUnoccupiedFieldsManager().getPartFields();
    }

    @Override
    public Collection<PartField> findImportedFields(FieldPlan fieldPlan) {
        FieldPlanManager fpm = getManagerForFieldPlan(fieldPlan);
        return fpm.getUnoccupiedFieldsManager().getPartFields();
    }
        
    @Override
    public ProductionUnit importOperations(FieldPlan fieldplan, List<Operation> operations) {
        ProductionUnit importUnit = ProductionUnit.IMPORT;
        FieldPlanManager fieldPlanManager = getManagerForFieldPlan(fieldplan);
        //TODO check if there is already an import production unit
        ProductionUnitManager importManager = fieldPlanManager.addProductionUnit(importUnit);
        for (Operation operation:operations){
            importManager.addOperation(operation);
        }
        return importUnit;
    }

    /**
     * Root node for the fieldplan explorer
     * Contains subnodes for all available fieldplans
     * 
     */
    class FieldPlanRootNode extends AbstractNode {

        public FieldPlanRootNode() throws IntrospectionException {
            super(Children.create(fieldPlanNodeFactory, true));
            setIconBaseWithExtension("nl/cloudfarming/client/icon/plant-icon.png");
        }

        @Override
        public Action[] getActions(boolean context) {
            return new Action[]{new NewFieldPlanAction()};
        }
    }

    /**
     * Childfactory for the fieldplan explorer root node
     * Creates childnodes for all available fieldplans
     * 
     */
    class FieldPlanNodeFactory extends ChildFactory<FieldPlanManager> {

        @Override
        protected Node createNodeForKey(FieldPlanManager key) {
            return key.getRootNode();
        }

        @Override
        protected boolean createKeys(List<FieldPlanManager> toPopulate) {
            toPopulate.addAll(fieldPlanMap.values());
            return true;
        }

        void refresh() {
            refresh(true);
        }
    }
}
