/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical.wizard;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.text.MessageFormat;
import java.util.NoSuchElementException;
import javax.swing.AbstractAction;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningModule;
import nl.cloudfarming.client.planning.wizard.PlanningWizardAction;
import org.openide.DialogDisplayer;
import org.openide.WizardDescriptor;
import org.openide.WizardDescriptor.Panel;
import org.openide.util.NbBundle;

/**
 * Action which displays {@link ActionSelectionPanel}
 * it Listens to {@link ActionSelectionPanel} and if a type of wizard is chosen the wizard content will be updated
 * and the panels for that particular wizard will become available
 * @author Merijn Zengers
 */
public class PlanningWizard extends AbstractAction implements WizardDescriptor.Iterator, ChangeListener {
   
    private int index;
    private WizardDescriptor wizardDescriptor;
    private static final String TITLE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.planning_wizard_action.title");
    private static final String WIZARD_NAME = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.select.name");
    private ActionSelectionController actionSelectionController;

    public PlanningWizard() {
        init();
    }

    protected final void init() {
        putValue(NAME, TITLE);
        actionSelectionController = new ActionSelectionController();
        actionSelectionController.addChangeListener(this);
        index = 0;
        wizardDescriptor = new WizardDescriptor(this);
    }

    /**
     * Fired when the action is performed. Shows the dialog.
     * @param e 
     */
    @Override
    public void actionPerformed(ActionEvent e) {
        // {0} will be replaced by WizardDescriptor.Panel.getComponent().getName()
        wizardDescriptor.setTitleFormat(new MessageFormat("{0}"));
        wizardDescriptor.setTitle(TITLE);
        
        // Disable Finish-button for Action-selection panel. 
        // FIX: Doesn't work if dialog is already visible. 
        // A better place would probably be in getWizardContent()
        ((JComponent) wizardDescriptor.getOptions()[2]).setEnabled(false); 

        Dialog dialog = DialogDisplayer.getDefault().createDialog(wizardDescriptor);
        
        dialog.setVisible(true);
        dialog.toFront();

        boolean cancelled = wizardDescriptor.getValue() != WizardDescriptor.FINISH_OPTION;
        if (!cancelled) {
            getSelectedWizard().save(wizardDescriptor);
        }
        if(getSelectedWizard() != null){
           getSelectedWizard().clear(wizardDescriptor);
        }
        // TODO: [Merijn] Add panels = null to documentation

    }

    /**
     * Gets the content displayed in the wizard.  
     * @return WizardDescriptor.Panel with all the panels in it for the selected wizard
     */
    protected WizardDescriptor.Panel[] getWizardContent() {
        // if wizard is empty: show first panel (pick your choise of actions)
        if (getSelectedWizard() == null || getSelectedWizard().getPanels() == null) {
            return new WizardDescriptor.Panel[]{actionSelectionController};
        }
        WizardDescriptor.Panel[] content = new WizardDescriptor.Panel[getSelectedWizard().getPanels().length + 1];
        content[0] = actionSelectionController;
        System.arraycopy(getSelectedWizard().getPanels(), 0, content, 1, getSelectedWizard().getPanels().length);
        return content;
    }

    /**
     * Set all the clientProperties on the panels the selected wizard displays
     */
    protected void setPanelClientProperties() {
        JComponent[] components = new JComponent[getWizardContent().length];
        int i = 0;
        for (WizardDescriptor.Panel panel : getWizardContent()) {
            components[i] = (JComponent) panel.getComponent();
            i++;
        }
        int j = 0;
        for (JComponent component : components) {
            component.putClientProperty("WizardPanel_contentSelectedIndex", new Integer(j));
            // Sets steps names for a panel
            component.putClientProperty("WizardPanel_contentData", getWizardStepNames());
            // Turn on subtitle creation on each step
            component.putClientProperty("WizardPanel_autoWizardStyle", Boolean.TRUE);
            // Show steps on the left side with the image on the background
            component.putClientProperty("WizardPanel_contentDisplayed", Boolean.TRUE);
            // Turn on numbering of all steps
            component.putClientProperty("WizardPanel_contentNumbered", Boolean.TRUE);
            j++;
        }

    }

    /**
     * get the names of all the steps of the wizard.
     * @return String[] array with all the names of the steps of the wizard.
     */
    protected String[] getWizardStepNames() {
        String[] steps = new String[getWizardContent().length];
        int i = 0;
        for (WizardDescriptor.Panel panel : getWizardContent()) {
            Component c = panel.getComponent();
            steps[i] = c.getName();
            i++;
        }
        return steps;
    }

    /**
     * Gets the selected wizard. Will base this on the property @Link{SELECTED_ACTION}
     * @return the selected PlanningWizardAction.
     */
    protected PlanningWizardAction getSelectedWizard() {
        return (PlanningWizardAction) wizardDescriptor.getProperty(ActionSelectionController.SELECTED_ACTION);
    }

    /**
     * The current active panel
     * @return Panel the current active
     */
    @Override
    public Panel current() {
        return getWizardContent()[index];
    }

    /**
     * gets the name of the wizard
     * @return the name
     */
    @Override
    public String name() {
        return WIZARD_NAME;
    }

    /**
     * Determines if the wizard has next panels
     * @return return true if so else false.
     */
    @Override
    public boolean hasNext() {
        return index < getWizardContent().length - 1;
    }

    /**
     * Determines if the wizard has previous panels
     * @return boolean if so else false
     */
    @Override
    public boolean hasPrevious() {
        return index > 0;
    }

    /**
     * Go to next panel
     */
    @Override
    public void nextPanel() {
        if (!hasNext()) {
            throw new NoSuchElementException();
        }
        index++;
        wizardDescriptor.putProperty(WizardDescriptor.PROP_CONTENT_SELECTED_INDEX, index);
    }

    /**
     * Go to previous panel
     */
    @Override
    public void previousPanel() {
        if (!hasPrevious()) {
            throw new NoSuchElementException();
        }
        index--;
        wizardDescriptor.putProperty(WizardDescriptor.PROP_CONTENT_SELECTED_INDEX, index);
    }

    @Override
    public void addChangeListener(ChangeListener l) {
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
    }

    /**
     * Listener to the JComboBox on ActionSelectionPanel.
     * If triggered the property @LINK{SELECTED_ACTION} is set and the panel client properties are set. 
     * @param e 
     */
    @Override
    public void stateChanged(ChangeEvent e) {
        if (e.getSource() instanceof JComboBox) {
            JComboBox selected = (JComboBox) e.getSource();
            if (selected.getSelectedItem() != null) {
                wizardDescriptor.putProperty(ActionSelectionController.SELECTED_ACTION, selected.getSelectedItem());
            }
        }
        setPanelClientProperties();

        if (getSelectedWizard() != null) {
            wizardDescriptor.setTitle(getSelectedWizard().getWizardName());
        }
    }
}
