/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.planning.tactical.wizard.spray;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.model.Remedy;
import nl.cloudfarming.client.planning.tactical.TacticalPlanningModule;
import org.openide.util.NbBundle;

/**
 * Panel which will present a list of Crop Protection Pesticides. A user can choose at least one or more and continue the wizard.
 * @author Merijn Zengers
 */
public class RemedyPanel extends JPanel {

    private static final int NUMBER_OF_COLUMNS = 4;
    private static final String TABLEMODEL_COLUMN_ID = "ID";
    private static final String TABLEMODEL_COLUMN_AUTH_ID = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_id");
    private static final String TABLEMODEL_COLUMN_PESTICIDE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_pesticide");
    private static final String TABLEMODEL_COLUMN_APPLICATION = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_application");
    private static final String TABLEMODEL_COLUMN_PCT = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.crop_header_pct");
    private static final String TITLE = NbBundle.getMessage(TacticalPlanningModule.class, "field_plan.wizard.spray.crop_protection_list_panel.title");
    private JScrollPane scrollPane = new JScrollPane();
    private JTable cropProtectionTable = new JTable();
    private DefaultTableModel availableTableModel;
    private final Map<Integer, Remedy> remedyMap = new HashMap<Integer, Remedy>();

    /** Creates new form PlanningVisualPanel2 */
    public RemedyPanel() {
        initComponents();
    }

    @Override
    public String getName() {
        return TITLE;
    }

    /** 
     * This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        setLayout(new MigLayout("wrap 1"));

        cropProtectionTable.setRowSelectionAllowed(true);
        scrollPane.setViewportView(cropProtectionTable);

        this.add(scrollPane);
    }

    public void setAvailableRemedies(List<Remedy> availableCropProtections) {

        if (!availableCropProtections.isEmpty()) {
            //Only if the table is empty
            if (availableTableModel == null) {
                String[][] displayList = new String[availableCropProtections.size()][NUMBER_OF_COLUMNS];

                int i = 0;
                for (Remedy remedy : availableCropProtections) {
                    remedyMap.put(i, remedy);
                    displayList[i][0] = String.valueOf(remedy.getAuthorizationNumber());
                    displayList[i][1] = remedy.getAgentName();
                    displayList[i][2] = remedy.getApplicationMethod();
                    int safetyTerm = remedy.getSafetyTerm();
                    displayList[i][3] = String.valueOf(safetyTerm);
                    
                    
                    i++;
                }

                availableTableModel = new DefaultTableModel(displayList,
                        new String[]{
                            TABLEMODEL_COLUMN_AUTH_ID, TABLEMODEL_COLUMN_PESTICIDE, TABLEMODEL_COLUMN_APPLICATION, TABLEMODEL_COLUMN_PCT
                        });
                
                cropProtectionTable.setModel(availableTableModel);
                
                // Default row sorter
                CropProtectionTableRowSorter cropProtectionSorter = new CropProtectionTableRowSorter(cropProtectionTable.getModel());
                cropProtectionSorter.toggleSortOrder(3);
                cropProtectionSorter.toggleSortOrder(3); //Reverse
                cropProtectionTable.setRowSorter(cropProtectionSorter);
        
               }
        }
    }

    /**
     * Get the selected crop protection agents from the table.
     * @return String[rowIndex][colIndex] contains the selected cropProtections
     * rowIndex: The number of selected rows in the displayed table
     * colIndex: Fixed NUMBER_OF_COLUMNS
     */
    public List<Remedy> getSelectedRemedies() {
        List<Remedy> remedies = new ArrayList<Remedy>();

        for (int index : cropProtectionTable.getSelectedRows()) {
            int modelRow = cropProtectionTable.convertRowIndexToModel(index);
            Remedy remedy = remedyMap.get(modelRow);
            remedies.add(remedy);
        }
        return remedies;
    }
    
    private static class CropProtectionTableRowSorter extends TableRowSorter{

        public CropProtectionTableRowSorter(TableModel tableModel){
            super(tableModel);
        }
        
        @Override
        public Comparator getComparator(int column) {
            // Safety Term
            if(column == 3){
                return new Comparator(){

                    @Override
                    public int compare(Object o1, Object o2) {
                        // We are sure objects specified in the argument are Strings since column 4 contains Strings
                        Integer safetyTerm1 = Integer.parseInt((String) o1);
                        Integer safetyTerm2 = Integer.parseInt((String) o2);
                        
                        if(safetyTerm1 < safetyTerm2){
                            return -1;
                        }
                        else if(safetyTerm1 > safetyTerm2){
                            return 1;
                        }
                        else{
                            return 0;
                        }
                    }
                };
            }  
            
            // Authorization ID
            if(column == 0){
                return new Comparator(){
                    @Override
                    public int compare(Object o1, Object o2) {
                        // We are sure objects specified in the argument are Strings since column 4 contains Strings
                        Integer authId1 = Integer.parseInt((String) o1);
                        Integer authId2 = Integer.parseInt((String) o2);
                        
                        if(authId1 < authId2){
                            return -1;
                        }
                        else if(authId1 > authId2){
                            return 1;
                        }
                        else{
                            return 0;
                        }
                    }
                };
            } 
            // Default does a fine job on Strings
            return super.getComparator(column);
        }
    }
}